type_codes = {
    16: "bool",
    17: "bytea",
    18: "char",
    19: "name",
    20: "int8",
    21: "int2",
    22: "int2vector",
    23: "int4",
    24: "regproc",
    25: "text",
    26: "oid",
    27: "tid",
    28: "xid",
    29: "cid",
    30: "oidvector",
    71: "pg_type",
    75: "pg_attribute",
    81: "pg_proc",
    83: "pg_class",
    114: "json",
    142: "xml",
    194: "pg_node_tree",
    3361: "pg_ndistinct",
    3402: "pg_dependencies",
    5017: "pg_mcv_list",
    32: "pg_ddl_command",
    5069: "xid8",
    600: "point",
    601: "lseg",
    602: "path",
    603: "box",
    604: "polygon",
    628: "line",
    700: "float4",
    701: "float8",
    705: "unknown",
    718: "circle",
    790: "money",
    829: "macaddr",
    869: "inet",
    650: "cidr",
    774: "macaddr8",
    1033: "aclitem",
    1042: "bpchar",
    1043: "varchar",
    1082: "date",
    1083: "time",
    1114: "timestamp",
    1184: "timestamptz",
    1186: "interval",
    1266: "timetz",
    1560: "bit",
    1562: "varbit",
    1700: "numeric",
    1790: "refcursor",
    2202: "regprocedure",
    2203: "regoper",
    2204: "regoperator",
    2205: "regclass",
    4191: "regcollation",
    2206: "regtype",
    4096: "regrole",
    4089: "regnamespace",
    2950: "uuid",
    3220: "pg_lsn",
    3614: "tsvector",
    3642: "gtsvector",
    3615: "tsquery",
    3734: "regconfig",
    3769: "regdictionary",
    3802: "jsonb",
    4072: "jsonpath",
    2970: "txid_snapshot",
    5038: "pg_snapshot",
    3904: "int4range",
    3906: "numrange",
    3908: "tsrange",
    3910: "tstzrange",
    3912: "daterange",
    3926: "int8range",
    4451: "int4multirange",
    4532: "nummultirange",
    4533: "tsmultirange",
    4534: "tstzmultirange",
    4535: "datemultirange",
    4536: "int8multirange",
    2249: "record",
    2287: "_record",
    2275: "cstring",
    2276: "any",
    2277: "anyarray",
    2278: "void",
    2279: "trigger",
    3838: "event_trigger",
    2280: "language_handler",
    2281: "internal",
    2283: "anyelement",
    2776: "anynonarray",
    3500: "anyenum",
    3115: "fdw_handler",
    325: "index_am_handler",
    3310: "tsm_handler",
    269: "table_am_handler",
    3831: "anyrange",
    5077: "anycompatible",
    5078: "anycompatiblearray",
    5079: "anycompatiblenonarray",
    5080: "anycompatiblerange",
    4537: "anymultirange",
    4538: "anycompatiblemultirange",
    4600: "pg_brin_bloom_summary",
    4601: "pg_brin_minmax_multi_summary",
    1000: "_bool",
    1001: "_bytea",
    1002: "_char",
    1003: "_name",
    1016: "_int8",
    1005: "_int2",
    1006: "_int2vector",
    1007: "_int4",
    1008: "_regproc",
    1009: "_text",
    1028: "_oid",
    1010: "_tid",
    1011: "_xid",
    1012: "_cid",
    1013: "_oidvector",
    210: "_pg_type",
    270: "_pg_attribute",
    272: "_pg_proc",
    273: "_pg_class",
    199: "_json",
    143: "_xml",
    271: "_xid8",
    1017: "_point",
    1018: "_lseg",
    1019: "_path",
    1020: "_box",
    1027: "_polygon",
    629: "_line",
    1021: "_float4",
    1022: "_float8",
    719: "_circle",
    791: "_money",
    1040: "_macaddr",
    1041: "_inet",
    651: "_cidr",
    775: "_macaddr8",
    1034: "_aclitem",
    1014: "_bpchar",
    1015: "_varchar",
    1182: "_date",
    1183: "_time",
    1115: "_timestamp",
    1185: "_timestamptz",
    1187: "_interval",
    1270: "_timetz",
    1561: "_bit",
    1563: "_varbit",
    1231: "_numeric",
    2201: "_refcursor",
    2207: "_regprocedure",
    2208: "_regoper",
    2209: "_regoperator",
    2210: "_regclass",
    4192: "_regcollation",
    2211: "_regtype",
    4097: "_regrole",
    4090: "_regnamespace",
    2951: "_uuid",
    3221: "_pg_lsn",
    3643: "_tsvector",
    3644: "_gtsvector",
    3645: "_tsquery",
    3735: "_regconfig",
    3770: "_regdictionary",
    3807: "_jsonb",
    4073: "_jsonpath",
    2949: "_txid_snapshot",
    5039: "_pg_snapshot",
    3905: "_int4range",
    3907: "_numrange",
    3909: "_tsrange",
    3911: "_tstzrange",
    3913: "_daterange",
    3927: "_int8range",
    6150: "_int4multirange",
    6151: "_nummultirange",
    6152: "_tsmultirange",
    6153: "_tstzmultirange",
    6155: "_datemultirange",
    6157: "_int8multirange",
    1263: "_cstring",
    12001: "pg_attrdef",
    12000: "_pg_attrdef",
    12003: "pg_constraint",
    12002: "_pg_constraint",
    12005: "pg_inherits",
    12004: "_pg_inherits",
    12007: "pg_index",
    12006: "_pg_index",
    12009: "pg_operator",
    12008: "_pg_operator",
    12011: "pg_opfamily",
    12010: "_pg_opfamily",
    12013: "pg_opclass",
    12012: "_pg_opclass",
    12015: "pg_am",
    12014: "_pg_am",
    12017: "pg_amop",
    12016: "_pg_amop",
    12019: "pg_amproc",
    12018: "_pg_amproc",
    12021: "pg_language",
    12020: "_pg_language",
    12023: "pg_largeobject_metadata",
    12022: "_pg_largeobject_metadata",
    12025: "pg_largeobject",
    12024: "_pg_largeobject",
    12027: "pg_aggregate",
    12026: "_pg_aggregate",
    12029: "pg_statistic",
    12028: "_pg_statistic",
    12031: "pg_statistic_ext",
    12030: "_pg_statistic_ext",
    12033: "pg_statistic_ext_data",
    12032: "_pg_statistic_ext_data",
    12035: "pg_rewrite",
    12034: "_pg_rewrite",
    12037: "pg_trigger",
    12036: "_pg_trigger",
    12039: "pg_event_trigger",
    12038: "_pg_event_trigger",
    12041: "pg_description",
    12040: "_pg_description",
    12043: "pg_cast",
    12042: "_pg_cast",
    12045: "pg_enum",
    12044: "_pg_enum",
    12047: "pg_namespace",
    12046: "_pg_namespace",
    12049: "pg_conversion",
    12048: "_pg_conversion",
    12051: "pg_depend",
    12050: "_pg_depend",
    1248: "pg_database",
    12052: "_pg_database",
    12054: "pg_db_role_setting",
    12053: "_pg_db_role_setting",
    12056: "pg_tablespace",
    12055: "_pg_tablespace",
    2842: "pg_authid",
    12057: "_pg_authid",
    2843: "pg_auth_members",
    12058: "_pg_auth_members",
    12060: "pg_shdepend",
    12059: "_pg_shdepend",
    12062: "pg_shdescription",
    12061: "_pg_shdescription",
    12064: "pg_ts_config",
    12063: "_pg_ts_config",
    12066: "pg_ts_config_map",
    12065: "_pg_ts_config_map",
    12068: "pg_ts_dict",
    12067: "_pg_ts_dict",
    12070: "pg_ts_parser",
    12069: "_pg_ts_parser",
    12072: "pg_ts_template",
    12071: "_pg_ts_template",
    12074: "pg_extension",
    12073: "_pg_extension",
    12076: "pg_foreign_data_wrapper",
    12075: "_pg_foreign_data_wrapper",
    12078: "pg_foreign_server",
    12077: "_pg_foreign_server",
    12080: "pg_user_mapping",
    12079: "_pg_user_mapping",
    12082: "pg_foreign_table",
    12081: "_pg_foreign_table",
    12084: "pg_policy",
    12083: "_pg_policy",
    12086: "pg_replication_origin",
    12085: "_pg_replication_origin",
    12088: "pg_default_acl",
    12087: "_pg_default_acl",
    12090: "pg_init_privs",
    12089: "_pg_init_privs",
    12092: "pg_seclabel",
    12091: "_pg_seclabel",
    4066: "pg_shseclabel",
    12093: "_pg_shseclabel",
    12095: "pg_collation",
    12094: "_pg_collation",
    12097: "pg_partitioned_table",
    12096: "_pg_partitioned_table",
    12099: "pg_range",
    12098: "_pg_range",
    12101: "pg_transform",
    12100: "_pg_transform",
    12103: "pg_sequence",
    12102: "_pg_sequence",
    12105: "pg_publication",
    12104: "_pg_publication",
    12107: "pg_publication_rel",
    12106: "_pg_publication_rel",
    6101: "pg_subscription",
    12108: "_pg_subscription",
    12110: "pg_subscription_rel",
    12109: "_pg_subscription_rel",
    12219: "pg_roles",
    12218: "_pg_roles",
    12224: "pg_shadow",
    12223: "_pg_shadow",
    12229: "pg_group",
    12228: "_pg_group",
    12233: "pg_user",
    12232: "_pg_user",
    12237: "pg_policies",
    12236: "_pg_policies",
    12242: "pg_rules",
    12241: "_pg_rules",
    12247: "pg_views",
    12246: "_pg_views",
    12252: "pg_tables",
    12251: "_pg_tables",
    12257: "pg_matviews",
    12256: "_pg_matviews",
    12262: "pg_indexes",
    12261: "_pg_indexes",
    12267: "pg_sequences",
    12266: "_pg_sequences",
    12272: "pg_stats",
    12271: "_pg_stats",
    12277: "pg_stats_ext",
    12276: "_pg_stats_ext",
    12282: "pg_stats_ext_exprs",
    12281: "_pg_stats_ext_exprs",
    12287: "pg_publication_tables",
    12286: "_pg_publication_tables",
    12292: "pg_locks",
    12291: "_pg_locks",
    12296: "pg_cursors",
    12295: "_pg_cursors",
    12300: "pg_available_extensions",
    12299: "_pg_available_extensions",
    12304: "pg_available_extension_versions",
    12303: "_pg_available_extension_versions",
    12309: "pg_prepared_xacts",
    12308: "_pg_prepared_xacts",
    12314: "pg_prepared_statements",
    12313: "_pg_prepared_statements",
    12318: "pg_seclabels",
    12317: "_pg_seclabels",
    12323: "pg_settings",
    12322: "_pg_settings",
    12329: "pg_file_settings",
    12328: "_pg_file_settings",
    12333: "pg_hba_file_rules",
    12332: "_pg_hba_file_rules",
    12337: "pg_timezone_abbrevs",
    12336: "_pg_timezone_abbrevs",
    12341: "pg_timezone_names",
    12340: "_pg_timezone_names",
    12345: "pg_config",
    12344: "_pg_config",
    12349: "pg_shmem_allocations",
    12348: "_pg_shmem_allocations",
    12353: "pg_backend_memory_contexts",
    12352: "_pg_backend_memory_contexts",
    12357: "pg_stat_all_tables",
    12356: "_pg_stat_all_tables",
    12362: "pg_stat_xact_all_tables",
    12361: "_pg_stat_xact_all_tables",
    12367: "pg_stat_sys_tables",
    12366: "_pg_stat_sys_tables",
    12372: "pg_stat_xact_sys_tables",
    12371: "_pg_stat_xact_sys_tables",
    12376: "pg_stat_user_tables",
    12375: "_pg_stat_user_tables",
    12381: "pg_stat_xact_user_tables",
    12380: "_pg_stat_xact_user_tables",
    12385: "pg_statio_all_tables",
    12384: "_pg_statio_all_tables",
    12390: "pg_statio_sys_tables",
    12389: "_pg_statio_sys_tables",
    12394: "pg_statio_user_tables",
    12393: "_pg_statio_user_tables",
    12398: "pg_stat_all_indexes",
    12397: "_pg_stat_all_indexes",
    12403: "pg_stat_sys_indexes",
    12402: "_pg_stat_sys_indexes",
    12407: "pg_stat_user_indexes",
    12406: "_pg_stat_user_indexes",
    12411: "pg_statio_all_indexes",
    12410: "_pg_statio_all_indexes",
    12416: "pg_statio_sys_indexes",
    12415: "_pg_statio_sys_indexes",
    12420: "pg_statio_user_indexes",
    12419: "_pg_statio_user_indexes",
    12424: "pg_statio_all_sequences",
    12423: "_pg_statio_all_sequences",
    12429: "pg_statio_sys_sequences",
    12428: "_pg_statio_sys_sequences",
    12433: "pg_statio_user_sequences",
    12432: "_pg_statio_user_sequences",
    12437: "pg_stat_activity",
    12436: "_pg_stat_activity",
    12442: "pg_stat_replication",
    12441: "_pg_stat_replication",
    12447: "pg_stat_slru",
    12446: "_pg_stat_slru",
    12451: "pg_stat_wal_receiver",
    12450: "_pg_stat_wal_receiver",
    12455: "pg_stat_subscription",
    12454: "_pg_stat_subscription",
    12460: "pg_stat_ssl",
    12459: "_pg_stat_ssl",
    12464: "pg_stat_gssapi",
    12463: "_pg_stat_gssapi",
    12468: "pg_replication_slots",
    12467: "_pg_replication_slots",
    12473: "pg_stat_replication_slots",
    12472: "_pg_stat_replication_slots",
    12477: "pg_stat_database",
    12476: "_pg_stat_database",
    12482: "pg_stat_database_conflicts",
    12481: "_pg_stat_database_conflicts",
    12486: "pg_stat_user_functions",
    12485: "_pg_stat_user_functions",
    12491: "pg_stat_xact_user_functions",
    12490: "_pg_stat_xact_user_functions",
    12496: "pg_stat_archiver",
    12495: "_pg_stat_archiver",
    12500: "pg_stat_bgwriter",
    12499: "_pg_stat_bgwriter",
    12504: "pg_stat_wal",
    12503: "_pg_stat_wal",
    12508: "pg_stat_progress_analyze",
    12507: "_pg_stat_progress_analyze",
    12513: "pg_stat_progress_vacuum",
    12512: "_pg_stat_progress_vacuum",
    12518: "pg_stat_progress_cluster",
    12517: "_pg_stat_progress_cluster",
    12523: "pg_stat_progress_create_index",
    12522: "_pg_stat_progress_create_index",
    12528: "pg_stat_progress_basebackup",
    12527: "_pg_stat_progress_basebackup",
    12533: "pg_stat_progress_copy",
    12532: "_pg_stat_progress_copy",
    12538: "pg_user_mappings",
    12537: "_pg_user_mappings",
    12543: "pg_replication_origin_status",
    12542: "_pg_replication_origin_status",
    13669: "cardinal_number",
    13668: "_cardinal_number",
    13672: "character_data",
    13671: "_character_data",
    13674: "sql_identifier",
    13673: "_sql_identifier",
    13677: "information_schema_catalog_name",
    13676: "_information_schema_catalog_name",
    13680: "time_stamp",
    13679: "_time_stamp",
    13682: "yes_or_no",
    13681: "_yes_or_no",
    13686: "applicable_roles",
    13685: "_applicable_roles",
    13691: "administrable_role_authorizations",
    13690: "_administrable_role_authorizations",
    13695: "attributes",
    13694: "_attributes",
    13700: "character_sets",
    13699: "_character_sets",
    13705: "check_constraint_routine_usage",
    13704: "_check_constraint_routine_usage",
    13710: "check_constraints",
    13709: "_check_constraints",
    13715: "collations",
    13714: "_collations",
    13720: "collation_character_set_applicability",
    13719: "_collation_character_set_applicability",
    13725: "column_column_usage",
    13724: "_column_column_usage",
    13730: "column_domain_usage",
    13729: "_column_domain_usage",
    13735: "column_privileges",
    13734: "_column_privileges",
    13740: "column_udt_usage",
    13739: "_column_udt_usage",
    13745: "columns",
    13744: "_columns",
    13750: "constraint_column_usage",
    13749: "_constraint_column_usage",
    13755: "constraint_table_usage",
    13754: "_constraint_table_usage",
    13760: "domain_constraints",
    13759: "_domain_constraints",
    13765: "domain_udt_usage",
    13764: "_domain_udt_usage",
    13770: "domains",
    13769: "_domains",
    13775: "enabled_roles",
    13774: "_enabled_roles",
    13779: "key_column_usage",
    13778: "_key_column_usage",
    13784: "parameters",
    13783: "_parameters",
    13789: "referential_constraints",
    13788: "_referential_constraints",
    13794: "role_column_grants",
    13793: "_role_column_grants",
    13798: "routine_column_usage",
    13797: "_routine_column_usage",
    13803: "routine_privileges",
    13802: "_routine_privileges",
    13808: "role_routine_grants",
    13807: "_role_routine_grants",
    13812: "routine_routine_usage",
    13811: "_routine_routine_usage",
    13817: "routine_sequence_usage",
    13816: "_routine_sequence_usage",
    13822: "routine_table_usage",
    13821: "_routine_table_usage",
    13827: "routines",
    13826: "_routines",
    13832: "schemata",
    13831: "_schemata",
    13836: "sequences",
    13835: "_sequences",
    13841: "sql_features",
    13840: "_sql_features",
    13846: "sql_implementation_info",
    13845: "_sql_implementation_info",
    13851: "sql_parts",
    13850: "_sql_parts",
    13856: "sql_sizing",
    13855: "_sql_sizing",
    13861: "table_constraints",
    13860: "_table_constraints",
    13866: "table_privileges",
    13865: "_table_privileges",
    13871: "role_table_grants",
    13870: "_role_table_grants",
    13875: "tables",
    13874: "_tables",
    13880: "transforms",
    13879: "_transforms",
    13885: "triggered_update_columns",
    13884: "_triggered_update_columns",
    13890: "triggers",
    13889: "_triggers",
    13895: "udt_privileges",
    13894: "_udt_privileges",
    13900: "role_udt_grants",
    13899: "_role_udt_grants",
    13904: "usage_privileges",
    13903: "_usage_privileges",
    13909: "role_usage_grants",
    13908: "_role_usage_grants",
    13913: "user_defined_types",
    13912: "_user_defined_types",
    13918: "view_column_usage",
    13917: "_view_column_usage",
    13923: "view_routine_usage",
    13922: "_view_routine_usage",
    13928: "view_table_usage",
    13927: "_view_table_usage",
    13933: "views",
    13932: "_views",
    13938: "data_type_privileges",
    13937: "_data_type_privileges",
    13943: "element_types",
    13942: "_element_types",
    13948: "_pg_foreign_table_columns",
    13947: "__pg_foreign_table_columns",
    13953: "column_options",
    13952: "_column_options",
    13957: "_pg_foreign_data_wrappers",
    13956: "__pg_foreign_data_wrappers",
    13961: "foreign_data_wrapper_options",
    13960: "_foreign_data_wrapper_options",
    13965: "foreign_data_wrappers",
    13964: "_foreign_data_wrappers",
    13969: "_pg_foreign_servers",
    13968: "__pg_foreign_servers",
    13974: "foreign_server_options",
    13973: "_foreign_server_options",
    13978: "foreign_servers",
    13977: "_foreign_servers",
    13982: "_pg_foreign_tables",
    13981: "__pg_foreign_tables",
    13987: "foreign_table_options",
    13986: "_foreign_table_options",
    13991: "foreign_tables",
    13990: "_foreign_tables",
    13995: "_pg_user_mappings",
    13994: "__pg_user_mappings",
    14000: "user_mapping_options",
    13999: "_user_mapping_options",
    14005: "user_mappings",
    14004: "_user_mappings",
    1261458: "citext",
    1261463: "_citext",
    4382494: "circles",
    4382493: "_circles",
}
